#ifndef LEXICOGRAPHIC_H
#define	LEXICOGRAPHIC_H

#include "GlobalConstraint.h"

namespace Cosoco {

    class Lexicographic : public GlobalConstraint {
        
    public : 
        vec<Variable *> X,Y;
        bool strict;
        Lexicographic(Problem &p,std::string n,vec<Variable*> &X,vec<Variable*> &Y,bool st);
       
        virtual bool isCorrectlyDefined() override;

        
        // Filtering method, return false if a conflict occurs
        virtual bool filter(Variable *x) override ;
        
        // Checking
        virtual bool isSatisfiedBy(vec<int>& tuple) override;
    protected : 
        int alpha;

	vec<int> times; // times[posx] gives the time at which the variable x has been set (pseudo-assigned)
	vec<int> vals; // vals[posx] gives the val of the variable x set at time times[posx]
	int time;

        bool establishAC(Variable *x,Variable *y);
        bool isConsistentPair(Variable *x, int vx, Variable *y, int vy);
        void setTime(int posx,int v);
    };


    class LexicographicLT : public Lexicographic {
    public : 
        LexicographicLT (Problem &p,std::string n,vec<Variable*> &X,vec<Variable*> &Y) : Lexicographic(p,n,X,Y,true) {}
    };

    class LexicographicLE : public Lexicographic {
    public : 
        LexicographicLE (Problem &p,std::string n,vec<Variable*> &X,vec<Variable*> &Y) : Lexicographic(p,n,X,Y,false) {}
    };
}

#endif	/* LEXICOGRAPHIC_H */

