#include "ElementConstant.h"
#include "solver/Solver.h"

using namespace Cosoco;


//----------------------------------------------
// Check validity
//----------------------------------------------

bool ElementConstant::isSatisfiedBy(vec<int> &tuple) {
    int idx = tuple[tuple.size() - 1];
    return tuple[idx - (startAtOne ? 1 : 0)] == result;
}


//----------------------------------------------
// Filtering
//----------------------------------------------


bool ElementConstant::filter(Variable *dummy) {
    if(index->size() > 1) { // checking that the values of index are still valid
        for(int idv : reverse(index->domain)) {
            int v = index->domain.toVal(idv);
            if(getVariableFor(v)->containsValue(result) == false && solver->delVal(index, v) == false)
                return false;
        }
    }

    // Not else : domain can have changed
    if(index->size() == 1) {
        Variable *x = getVariableFor(index->value()); // This value must be equal to result
        return solver->assignToVal(x, result);
    }
    return true;
}


//----------------------------------------------
// Constructor and initialisation methods
//----------------------------------------------


ElementConstant::ElementConstant(Problem &p, std::string n, vec<Variable *> &vars, Variable *i, int kk, bool one) :
        Element(p, n, "Element Constant", vars.size() + 1, i, one), result(kk) {
    szVector = vars.size();

    int pos = vars.firstOccurrenceOf(i);
    if(pos != -1)
        throw std::runtime_error("element with index in list not yet supported");

    vars.push(i);
    scopeInitialisation(vars);
    vars.pop();
}
