#include "ExactlyK.h"
#include "solver/Solver.h"

using namespace Cosoco;


//----------------------------------------------
// Check validity and correct definition
//----------------------------------------------

bool ExactlyK::isSatisfiedBy(vec<int> &tuple) {
    int nb = 0;
    for(int v : tuple)
        if(v == value) nb++;
    return nb == k;

}


bool ExactlyK::isCorrectlyDefined() {
    if(k < 0 || k > scope.size())
        throw std::logic_error("Constraint " + std::to_string(idc) + ": ExactlyK must have 0 <= k <= size(list)");
    for(Variable *x : scope)
        if(x->containsValue(value) == false)
            throw std::logic_error("Constraint " + std::to_string(idc) + ": Exactly, all variables must contain value");
    return true;
}

//----------------------------------------------
// Check validity and correct definition
//----------------------------------------------

bool ExactlyK::filter(Variable *dummy) {
    int nbGuaranteedOccurrences = 0, nbPossibleOccurrences = 0;


    if(k == 0) {
        for(int posx : unassignedVariablesIdx)
            if(solver->delVal(scope[posx], value) == false)
                return false;
        done = true;
    }

    for(Variable *x : scope) {
        if(x->containsValue(value)) {
            nbPossibleOccurrences++;
            if(x->size() == 1 && ++nbGuaranteedOccurrences > k)
                return false;
        }
    }

    if(nbGuaranteedOccurrences == k) {
        for(int posx : unassignedVariablesIdx) {
            Variable *x = scope[posx];
            if(x->size() > 1 && solver->delVal(x, value) == false) // delvalue can not be false ?
                return false;
        }
        return true;
    }

    if(nbPossibleOccurrences < k)
        return false;
    if(nbPossibleOccurrences == k) {
        for(int posx : unassignedVariablesIdx) {
            Variable *x = scope[posx];
            if(x->size() > 1 && x->containsValue(value) && solver->assignToVal(x, value) == false)
                return false;
        }
    }
    return true;
}

//----------------------------------------------
// Construction and initialisation
//----------------------------------------------

ExactlyK::ExactlyK(Problem &p, std::string n, vec<Variable *> &vars, int kk, int val) : GlobalConstraint(p, n, "Exactly K", vars), k(kk), value(val), done(false) {
}


State ExactlyK::status() { return done ? CONSISTENT : UNDEF; }


void ExactlyK::reinitialize() { done = false; }
