#include "EQ.h"
#include "solver/Solver.h"

using namespace Cosoco;

//----------------------------------------------
// Check validity and correct definition
//----------------------------------------------

bool EQ::isSatisfiedBy(vec<int> &tuple) {
    return tuple[0] == tuple[1];
}

//----------------------------------------------
// Filtering
//----------------------------------------------

bool EQ::filter(Variable *xx) {

    Variable *hasChanged;
    Variable *toChange;
    int pos;

    if(xx->idx == x->idx) {
        hasChanged = x;
        toChange = y;
        pos = (lastx == -1 || lastx <= x->size()) ? x->domain.maxSize() : lastx;
    } else {
        hasChanged = y;
        toChange = x;
        pos = (lasty == -1 || lasty <= y->size()) ? y->domain.maxSize() : lasty;
    }


    if(hasChanged->size() == 1) {
        if(solver->assignToVal(toChange, hasChanged->value()) == false)
            return false;
        lastx = -1;
        lasty = -1;
        return true;
    } else {
        for(int i = hasChanged->domain.size() ; i < pos ; i++) { // Check removed variables
            int v = hasChanged->domain.valueAtPosition(i);
            if(toChange->containsValue(v) == false && solver->delVal(toChange, v) == false)
                return false;
        }
    }

    if(xx->idx == x->idx)
        lastx = x->domain.size();
    else
        lasty = y->domain.size();

    return true;
}



//----------------------------------------------
// Construction and initialisation
//----------------------------------------------

EQ::EQ(Problem &p, std::string n, Variable *xx, Variable *yy) : Binary(p, n, xx, yy), lastx(-1), lasty(-1) { type = "(X = Y)"; }
