#ifndef OPTIMIZER_H
#define    OPTIMIZER_H

#include <solver/heuristics/values/ForceIdv.h>
#include "solver/heuristics/variables/HeuristicVarLNS.h"
#include "solver/AbstractSolver.h"
#include "ObjectiveConstraint.h"
#include "core/Problem.h"
#include "solver/Solver.h"
#include "core/OptimizationProblem.h"
#include "Solution.h"

namespace Cosoco {


    class Optimizer : public AbstractSolver {
    protected :

        long lower, upper;                                  // The current lower et upper bound
        Solution bestSolution;                              // The solution manager (used to avoid problems if the solver is killed during solution storing
        long best;                                          // Best value until now
    public :
        bool boundFound;                                    // One solution has been found?
        bool invertBestCost;                                // usefull with sum and minimize, we invert the cost due to SumGE implementation only
        int callToSolver;                                   // The nb calls to the SAT solver
        Solver *solver;                                     // The solver used to optimize
        ObjectiveConstraint *objectiveLB;                   // ctr >= k
        ObjectiveConstraint *objectiveUB;                   // ctr <= k
        OptimisationType optimtype;                         // Minimize or Maximize
        bool useDicothomicMode;                             // TODO: class inheritence ?
        bool progressSaving;
        bool lns;
        int fragmentSizeInLNS;
        Optimizer();

        void solve() override;
        void printFinalStats() override;
        void displayCurrentSolution() override;

        void solveInDichotomicMode();
        void solveInOneDirection();

        void setSolver(Solver *s);


        long bestCost() {
            assert(nbSolutions > 0);
            return (invertBestCost ? -1 : 1) * best;
        }


        bool hasSolution() override { return bestSolution.exists(); }


        void addProgressSaving() {
            assert(solver != nullptr);
            progressSaving = true;
            solver->heuristicVal = new ForceIdvs(*solver, solver->heuristicVal);
        }


        void addLNS() {
            assert(solver != nullptr);
            lns = true;
            fragmentSizeInLNS = solver->problem.nbVariables() / 5;
            solver->heuristicVal = new ForceIdvs(*solver, solver->heuristicVal, false);
            solver->heuristicVar = new HeuristicVarLNS(*solver, solver->heuristicVar);
        }
    };
}

#endif    /* OPTIMIZER_H */

