#ifndef COSOCO_SOLUTION_H
#define COSOCO_SOLUTION_H

#include <map>
#include <iostream>
#include <XCSP3Constants.h>
#include "mtl/Vec.h"


namespace Cosoco {
    class Solution {  // This class comes from pseudo boolean competition and was provided by O. Roussel.
    private:
        std::map<std::string, int> *tmp, *preserved;
        long bound;

    public:
        Solution() {
            tmp = preserved = nullptr;
        }


        /**
         * to be called when a new solution will be entered
         */
        void begin(long b) {
            bound = b;
            tmp = new std::map<std::string, int>;
        }


        /**
         * append a literal to the new solution being entered
         */
        void appendTo(const std::string &variable, int value) {
            (*tmp)[variable] = value;
        }


        /**
         * to be called once a solution is completely entered
         */
        void end() {
            std::map<std::string, int> *del = preserved;
            // begin critical section
            preserved = tmp;
            // end critical section
            tmp = nullptr;
            if(del)
                delete del;
        }


        /**
         * return true is a solution was recorded
         */
        bool exists() {
            return preserved != nullptr;
        }


        /**
         * return the latest stable solution
         *
         * even in case of an interrupt, the returned solution is coherent as long
         * as endSolution() is not called in between
         */
        const std::map<std::string, int> &get() const {
            if(!preserved)
                throw std::runtime_error("no solution recorded");

            return *preserved;
        }


        void display() {
            if(exists() == false)
                throw std::runtime_error("The solution does not exists");
            printf("\nv <instantiation type='solution' cost='%ld'>\n", bound);
            printf("v <list>");
            for(std::map<std::string, int>::iterator it = preserved->begin(); it != preserved->end(); ++it)
                std::cout << it->first << " ";
            printf("</list>\n");

            printf("v <values>");
            for(std::map<std::string, int>::iterator it = preserved->begin(); it != preserved->end(); ++it)
                if(it->second == STAR)
                    std::cout << "* ";
                else
                    std::cout << it->second << " ";
            printf("</values>\n");

            printf("v </instantiation>\n\n");
            std::cout << std::flush;
        }
    };
}
#endif //COSOCO_SOLUTION_H
