#ifndef ABSTRACTSOLVER_H
#define    ABSTRACTSOLVER_H

#include "utils/Verbose.h"

namespace Cosoco {

    enum STATE {
        RUNNING, REACH_GOAL, FULL_EXPLORATION, OPTIMUM
    };

    class AbstractSolver {
    public  :
        STATE status;                                       // The status of the solver
        bool displayModels;
        int nbSolutions;                                    // Number of solutions already found
        Verbose verbose;                                    // The level of verbose mode 0..3
        double random_seed;                                 // The seed used by the solver
        virtual void solve() = 0;
        virtual void printFinalStats() = 0;                 // The final stats to print
        virtual void printIntermediateStats() { }           // The intermediate stats to print

        virtual void displayCurrentSolution() = 0;          // display the current solution

        AbstractSolver() : status(RUNNING), displayModels(false), nbSolutions(0), random_seed(91648253) { }


        void setVerbosity(int vv) {
            Verbose v;
            verbose.verbosity = vv;
        }
        virtual bool hasSolution() {
            return nbSolutions > 0;
        }

    };
}


#endif	/* ABSTRACTSOLVER_H */

