#include "HeuristicVarDomWdeg.h"

using namespace Cosoco;


HeuristicVarDomWdeg::HeuristicVarDomWdeg(Solver &s) : HeuristicVar(s) {
    s.addObserverConflict(this);
}


Variable *HeuristicVarDomWdeg::select() {
    Variable *x = solver.decisionVariables[0];
    double bestV = ((double) x->size()) / x->wdeg;

    for (int i = 1 ; i < solver.decisionVariables.size() ; i++) {
        Variable *y = solver.decisionVariables[i];
        if (((double) y->size()) / y->wdeg < bestV) {
            bestV = ((double) y->size()) / y->wdeg;
            x = y;
        }

    }
    assert(x != nullptr);
    return x;
}


void HeuristicVarDomWdeg::notifyConflict(Constraint *c, int level) {
    c->wdeg++;
    int notThisposition = NOTINSCOPE;
    if (c->unassignedVariablesIdx.size() == 1) {
        notThisposition = c->unassignedVariablesIdx[0];//c->toScopePosition(c->unassignedVariablesIdx[0]);
    }

    for (int i = 0 ; i < c->scope.size() ; i++) {
        if (i == notThisposition) continue;
        c->scope[i]->wdeg++;
    }

}


void HeuristicVarDomWdeg::notifyNewDecision(Variable *x, Solver &s) {
    for (Constraint *c : x->constraints) {
        if (c->unassignedVariablesIdx.size() == 1)
            c->scope[c->unassignedVariablesIdx[0]]->wdeg -= c->wdeg;
    }
}


void HeuristicVarDomWdeg::notifyDeleteDecision(Variable *x, int v, Solver &s) {
    for (Constraint *c : x->constraints) {
        if (c->unassignedVariablesIdx.size() == 2)
            c->scope[c->unassignedVariablesIdx[0]]->wdeg += c->wdeg;
    }
}
