#include "HeuristicVarLNS.h"
#include <solver/Solver.h>
#include "utils/Utils.h"
#include "solver/heuristics/values/ForceIdv.h"

using namespace Cosoco;


HeuristicVarLNS::HeuristicVarLNS(Solver &s, HeuristicVar *h) : HeuristicVar(s), hvar(h), _stop(false), maxConflicts(1000) {
    idvs.growTo(solver.problem.nbVariables(), -1);
}


Variable *HeuristicVarLNS::select() {
    if(solver.decisionLevel() >= variables.size())
        return hvar->select();
    assert(variables.size() == 0 || solver.isAssigned(variables[solver.decisionLevel()]) == false);
    assert(variables.size() == 0 || variables[solver.decisionLevel()]->containsIdv(idvs[variables[solver.decisionLevel()]->idx]));
    assert(variables.size() == 0 || solver.heuristicVal->select(variables[solver.decisionLevel()]) != -1);
    return variables[solver.decisionLevel()];
}


void HeuristicVarLNS::setIdvs(vec<int> &ids) {
    ids.copyTo(idvs);
}


void HeuristicVarLNS::createFragment(int nbVariables) {
    _stop = false;
    for(Variable *x : solver.problem.variables)
        x->fake = false;
    variables.clear();
    vec<int> ids;
    ids.growTo(solver.problem.variables.size(), -1);
    for(int i = 0 ; i < nbVariables ;) {
        int idx = irand(solver.random_seed, solver.problem.variables.size());
        if(solver.problem.variables[idx]->fake || solver.isAssigned(solver.problem.variables[idx]) ||
           solver.problem.variables[idx]->containsIdv(idvs[idx]) == false)
            continue;
        i++;
        variables.push(solver.problem.variables[idx]);
        ids[idx] = idvs[idx];
        solver.problem.variables[idx]->fake = true;
    }
    ((ForceIdvs *) solver.heuristicVal)->setIdValues(ids);
    initialNbConflits = solver.conflicts;
}


bool HeuristicVarLNS::stop() {
    if(variables.size() == 0)
        return false;
    if(_stop) return true;
    if(solver.conflicts - initialNbConflits > maxConflicts) {// Too many conflicts
        _stop = true;
        return true;
    }
    if(solver.decisionLevel() > variables.size()) // is the fragment unsat
        return false;
    for(Variable *x : variables)
        if(x->containsIdv(idvs[x->idx]) == false) {
            _stop = true;
            return true;
        }
    return false;
}


bool HeuristicVarLNS::fragmentIsUnsat() {
    return _stop && solver.conflicts - initialNbConflits <= maxConflicts;
}


void HeuristicVarLNS::addFragmentToNogoods() {
    if(solver.satWrapper == nullptr)
        return;
    vec<CSPPropagation> nogood;
    if(variables.size() == 0) {// TODO: SERACH is DONE
        assert(false);
        return;
    }
    for(Variable *x : variables) {
        nogood.push();
        nogood.last().equal = false; // we store nogood : not(x=a or y=b) -> z!=c) <-> x!=a or y!=b or z!=c
        nogood.last().x = x;
        nogood.last().idv = idvs[x->idx];
    }

    solver.satWrapper->addClause(nogood, true);
}


